function WW = sweights(case_w, model,Data,varargin)


% ===================================================================
% NAME: sweights.m
% PURPOSE: Computes alternative weight matrices
%          depending on the model to estimate
%
% "Strategic Interaction in Tax Policies Among States"
% by Ruben Hernandez-Murillo. (2003)
%
% CALL: WW = sweights(case_w, model,Data,varargin)
% 		varargin={W0,W0text,BEA_Regions,Crone_Regions,gov_var}
% 
% Possible Models: (model_string)
% 1. Avg. Cap. Tax Rate.----- (actr) --------- N=49, T=23, dates=1977-1999
%
% Spatial Matrices: (case_string)
% 1.  Contiguity: Borders--------------------- (border)   (not standardized)
% 2.  Contiguity: BEA regions ---------------- (bea)      (not standardized)
% 3.  Contiguity: Crone's regions ------------ (crone)    (not standardized)
% 4.  Geo-Distance Decay (gravity) ----------- (gravity)  (not standardized)
% 5.  Features Distance Decay (Mahalanobis)--- (features) (not standardized)
% 6.  Population ----------------------------- (pop)      (not standardized)
% 7.  Population share: Borders -------------- (popborder)
% 8.  Population share: BEA regions ---------- (popbea)
% 9.  Population share: Crone's regions ------ (popcrone)
% 10. Income --------------------------------- (inc)      (not standardized)
% 11. Income share: Borders ------------------ (incborder)
% 12. Income share: BEA regions -------------- (incbea)
% 13. Income share: Crone's regions ---------- (inccrone)

% Matrices for gravity, features, population, and income are not 
% row-standardized at this point to allow for more combinations with border, 
% bea, and crone
%
% If using variables from the state public finances database (DatGov)
% sample frequency changes to 1977-1999, as in Avg_CTR.
%
% ===================================================================
% Note: Crone does not place DC in any Regions so neighbors(DC)=[]
% But DC is not dropped.
%
% REQUIRES: function mahaldist.m by Peter J. Acklam (2001).
% http://home.online.no/~pjacklam/matlab/index.html

W0=varargin{1};
W0text=varargin{2};
BEA_Regions=varargin{3};
Crone_Regions=varargin{4};
gov_var=varargin{5};			%Binary: 1=using government variables -> change 
								%sample frequency

matrix_case=[case_w,'-',model];

switch lower(matrix_case)

case {'border-actr'}
	disp('Matrix is Border')
	WZ=(W0);
	model=='actr'
	T=length(1977:1999); WW=wdiag({WZ},T);
case {'bea-actr'}
	disp('Matrix is BEA')
	WZ=(wregions(W0text,BEA_Regions));
	model=='actr'
	T=length(1977:1999); WW=wdiag({WZ},T);
case {'crone-actr'}
	disp('Matrix is Crone')
	WZ=(wregions(W0text,Crone_Regions));
	model=='actr'
	T=length(1977:1999); WW=wdiag({WZ},T);
case 'features-actr'
	disp('Matrix is Mahalanobis distance');
	Ft={[Data.Population_Density.var], [Data.Avg_Temperature.var], [Data.Percapita_RPI.var]};
	time0=Data.Population(1).time;
	time1=Data.Avg_CTR(1).time;
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(1977:1999);
		Feat=[Ft{1}(I(t),:)' Ft{2}(I(t),:)' Ft{3}(I(t),:)'];
		Wt=1./mahaldist(Feat,[],inv(cov(Feat)));
		Wt(find(Wt==Inf))=0;
		FF{t}=(Wt);
	end;
	WW=wdiag(FF,[]);
case 'gravity-actr'
	disp('Matrix is Decaying square Distance in Miles')
	Ft={[Data.Population_Center.lat], [Data.Population_Center.lon]};
	time0=Data.Population(1).time;
	time1=Data.Avg_CTR(1).time;
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=[Ft{1}(I(t),:)' Ft{2}(I(t),:)'];
		Wt=1./(dlat_long(Feat).^2);
		Wt(find(Wt==Inf))=0;
		FF{t}=(Wt);
	end;
	WW=wdiag(FF,[]);
	
case 'pop-actr'
	disp('Matrix is Population')
	time0=Data.Population(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Population.var]};
	nstates=length(W0text);
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat;
		FF{t}=(Wt);
	end;
	WW=wdiag(FF,[]);	


case 'popborder-actr'
	disp('Matrix is Border-Population')
	time0=Data.Population(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Population.var]};
	nstates=length(W0text);
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat.*W0;
		FF{t}=normw(Wt);
	end;
	WW=wdiag(FF,[]);	

case 'popbea-actr'
	disp('Matrix is Population-BEA')
	WZ=wregions(W0text,BEA_Regions);
	nstates=length(W0text);
	time0=Data.Population(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Population.var]};
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat.*WZ;
		FF{t}=normw(Wt);
	end;
	WW=wdiag(FF,[]);	

case 'popcrone-actr'
	disp('Matrix is Population-Crone')
	WZ=wregions(W0text,Crone_Regions);
	nstates=length(W0text);
	time0=Data.Population(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Population.var]};
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat.*WZ;
		FF{t}=normw(Wt);
	end;
	WW=wdiag(FF,[]);	

case 'inc-actr'
	disp('Matrix is Personal_Income')
	nstates=length(W0text);
	time0=Data.Personal_Income(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Personal_Income.var]};
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat;
		FF{t}=(Wt);
	end;
	WW=wdiag(FF,[]);	

	
case 'incborder-actr'
	disp('Matrix is Border-Personal_Income')
	nstates=length(W0text);
	time0=Data.Personal_Income(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Personal_Income.var]};
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat.*W0;
		FF{t}=normw(Wt);
	end;
	WW=wdiag(FF,[]);	

case 'incbea-actr'
	disp('Matrix is Income-BEA')
	WZ=wregions(W0text,BEA_Regions);
	nstates=length(W0text);
	time0=Data.Personal_Income(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Personal_Income.var]};
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat.*WZ;
		FF{t}=normw(Wt);
	end;
	WW=wdiag(FF,[]);	


case 'inccrone-actr'
	disp('Matrix is Income-Crone')
	nstates=length(W0text);
	WZ=wregions(W0text,Crone_Regions);
	time0=Data.Personal_Income(1).time;
	time1=Data.Avg_CTR(1).time;
	Ft={[Data.Personal_Income.var]};
	[junk I]=setdiff(time0,setdiff(time0,time1));
	for t=1:length(time1);
		Feat=kron(ones(nstates,1),Ft{1}(I(t),:));
		Wt=Feat.*WZ;
		FF{t}=normw(Wt);
	end;
	WW=wdiag(FF,[]);	

otherwise
	error('Model and Weight-scheme not recognized')

end;


