function [results, Dhat]=kelejian(Y,X,W)

% NAME: kelejian.m 
% PURPOSE: estimates the spatial autocorrelation model 
% of Kelejian + Robinson (1993) using also the nonlinear least-squares
% estimate of (sigma^2,lambda) from Kelejian + Prucha (1996).
%   Y is the dependent variable,
%   X is the matrix of regressors
%   W is the spatial weights matrix
% MODEL: Y=rho*W1*Y+X*beta+u, u=lambda*W2*u+e,
%
% CALL: results=kelejian(Y,X,W), 
%
%   results is a structure with
%   results.meth    = Kelejian and Robinson (1993)
%   results.rho 	= rho (scalar) 
% 	results.beta	= beta (kx1)
%	results.lambda	= lambda
%	results.sigma2	= sigma2 (scalar)
%	results.zstats 	= (rho,beta')'./sqrt(diag(Asyvar))
%	results.pvalues = (1-normcdf(abs(zstats)))*2. Two-tailed test p-values
%	results.AsyVar	= Asymptotic Variance of (rho,beta')';
%	results.r2	    = R-squared
%	results.yhat	= fitted values
%	results.resid	= ehat=y-yhat
%
% REQUIRES: The MathWorks Statistics Toolbox
%
% Ruben Hernandez-Murillo (2003)

tic, 
n=size(Y,1);

% 1.- Use 2SLS with instruments H to obtain residuals for K+P's (1996) NLS estimator. Choose as instruments the linearly independent columns of [X WX]

H0=[X,W*X];
[R d]=rref(H0);   

H=H0(:,d);

Wy=W*Y;
beta=H\Wy;
Wyhat=H*beta;

Z=[Wyhat X];
gammahat=Z\Y;
uhat=Y-Z*gammahat;


ub=W*uhat;
ubb=W*W*uhat;

Jn=[           2*uhat'*ub/n,      -ub'*ub/n,            1;
                2*ubb'*ub/n,    -ubb'*ubb/n,           trace(W'*W)/n;
     (uhat'*ubb + ub'*ub)/n,      -ub'*ubb/n,           0];
     
jn=[uhat'*uhat/n;
        ub'*ub/n;
      uhat'*ub/n];


% 2.- NLS estimator with initial estimates from overparameterized OLS
% the function nlinfit from the Statistics Toolbox can be replaced
% with some other non-linear least-squares function.

beta=Jn\jn;
if abs(beta(1))>1
    if sqrt(abs(beta(2)))>1
	    theta0(1)=.01;
	else
		theta0(1)=sqrt(abs(beta(2)));
	end;
else
	theta0(1)=beta(1);
end;
theta0(2)=beta(3);
theta0=theta0(:);


[theta1,res,jac]=nlinfit(Jn,jn,@fit_kp,theta0);

if abs(theta1(1))>1|theta1(2)<0
	theta1
	%error('Estimate of lambda outside of [-1,1] range or var(epsilon)<0')

end;


% 3.- Obtain GMM estimator

lambda=theta1(1);
A=(eye(size(W))-lambda*W);
AI=A\eye(size(A));

if abs(W-W')<sqrt(eps)
	DH=theta1(2)*H'*AI*AI'*H;
else
	B=(eye(size(W'))-lambda*W');
	BI=B\eye(size(B));
	DH=theta1(2)*H'*AI*BI*H;
end;

DHI=DH\eye(size(DH));

Dhat=H*DHI*H';
ZDZ=Z'*Dhat*Z;
ZDZI=ZDZ\eye(size(ZDZ));

gamma_gmm=ZDZI*Z'*Dhat*Y;

VZ=Z'*H*DHI*H'*Z;
VZI=VZ\eye(size(VZ));


% 4.- Prepare results structure

clear results
results.meth='Kelejian and Robinson (1993)';
results.rho=gamma_gmm(1);
results.beta=gamma_gmm(2:end);
results.lambda=theta1(1);
results.sigma2=theta1(2);
results.AsyVar=VZI;
results.zstats=gamma_gmm./sqrt(diag(VZI));
results.pvalues=(1-normcdf(abs(results.zstats)))*2;
%results.Sigmau = DH;
%results.Dhat = Dhat;

yhat=gamma_gmm(1)*Wy+X*gamma_gmm(2:end);
results.yhat = yhat;

r=Y-yhat;
results.resid=r;

r2= mean((yhat - mean(yhat)).^2)/mean((Y - mean(Y)).^2);
results.r2=r2;


time=toc;
disp(['Total time elapsed= ',num2str(time),' seconds']);


