function lm = lm_ols(y,x,W,ols_results)

% NAME: lm_ols.m. LM statistics for the spatial lag/autocorrelation model.
% PURPOSE: Computes LM statistic using OLS residuals from 
%          Anselin et al. (1996) RSUE 26:77-104.
% MODEL: Y=X*beta+u 
%
% CALL:  lm_ols(y,x,W,ols_results)
% y: dependent varialbe
% x: matrix of regressors
% W: matrix of spatial weights
% ols_results: structure from LeSage's ols.m command
%
% REQUIRES:- The MathWorks Statistics Toolbox
%          - Econometrics Toolbox by James P. LeSage (2001) 
%            available at http://www.spatial-econometrics.com/
% PROVIDES: lm structure with test statistics and p-values
%
% Ruben Hernandez-Murillo (2003)

e=ols_results.resid;
nobs= ols_results.nobs;
s2=e'*e/nobs;
beta=ols_results.beta;

if nobs < 10000
  [q r] = qr(x,0);
  xpxi = (r'*r)\eye(size(x,2));
else % use Cholesky for very large problems
  xpxi = (x'*x)\eye(size(x,2));
end;

M=eye(nobs) - x*xpxi*x';
T = trace(W*W + W'*W);
D = (W*x*beta)'*M*(W*x*beta);
Ry= e'*W*y/s2;
Re = e'*W*e/s2;
nJ = (D + T*s2)/s2;

%% Test for H0: rho=lambda=0, H1: rho~=0 or lambda~=0

lm.both.stat = (Ry - Re)^2/(nJ - T) + Re^2/T;
lm.both.prob = 1-chi2cdf(lm.both.stat,2);
lm.both.crit1 = 9.2103;

%% Test for H0: rho=0, H1: rho~=0

lm.rho.stat = ((Ry - Re)^2)/( nJ - T); 
lm.rho.prob = 1-chi2cdf(lm.rho.stat,1);
lm.rho.crit1 = 6.6349;


%% Test for H0: lambda=0, H1: lambda~=0

lm.lam.stat = ((Re - T/nJ*Ry)^2) / (T - T*T/nJ);
lm.lam.prob = 1-chi2cdf(lm.lam.stat,1);
lm.lam.crit1 = 6.6349;













